@@ -181,6 +181,8 @@ class InterfaceController(object):
         with pyroute2.IPRoute() as ipr:
             idx = ipr.link_lookup(ifname=interface.name)[0]
 
+            # Workaround for https://github.com/PyCQA/pylint/issues/8497
+            # pylint: disable=E1136, E1121
             link = ipr.get_links(idx)[0]
             current_state = link.get(consts.STATE)
 
@@ -195,7 +197,9 @@ class InterfaceController(object):
 
             self._addresses_up(interface, ipr, idx)
             self._routes_up(interface, ipr, idx)
-            self._rules_up(interface, ipr, idx)
+            # only the vip port updates the rules
+            if interface.if_type == consts.VIP:
+                self._rules_up(interface, ipr, idx)
 
             self._scripts_up(interface, current_state)
 
@@ -352,11 +356,10 @@ class InterfaceController(object):
                               **rule)
 
     def _scripts_up(self, interface, current_state):
-        if current_state == consts.IFACE_DOWN:
-            for script in interface.scripts[consts.IFACE_UP]:
-                LOG.debug("%s: Running command '%s'",
-                          interface.name, script[consts.COMMAND])
-                subprocess.check_output(script[consts.COMMAND].split())
+        for script in interface.scripts[consts.IFACE_UP]:
+            LOG.debug("%s: Running command '%s'",
+                      interface.name, script[consts.COMMAND])
+            subprocess.check_output(script[consts.COMMAND].split())
 
     def down(self, interface):
         LOG.info("Setting interface %s down", interface.name)
@@ -370,15 +373,19 @@ class InterfaceController(object):
         with pyroute2.IPRoute() as ipr:
             idx = ipr.link_lookup(ifname=interface.name)[0]
 
+            # Workaround for https://github.com/PyCQA/pylint/issues/8497
+            # pylint: disable=E1136, E1121
             link = ipr.get_links(idx)[0]
             current_state = link.get(consts.STATE)
 
             if current_state == consts.IFACE_UP:
-                for rule in interface.rules:
-                    rule[consts.FAMILY] = self._family(rule[consts.SRC])
-                    LOG.debug("%s: Deleting rule %s", interface.name, rule)
-                    self._ipr_command(ipr.rule, self.DELETE,
-                                      raise_on_error=False, **rule)
+                # only the vip port updates the rules
+                if interface.if_type == consts.VIP:
+                    for rule in interface.rules:
+                        rule[consts.FAMILY] = self._family(rule[consts.SRC])
+                        LOG.debug("%s: Deleting rule %s", interface.name, rule)
+                        self._ipr_command(ipr.rule, self.DELETE,
+                                          raise_on_error=False, **rule)
 
                 for route in interface.routes:
                     route[consts.FAMILY] = self._family(route[consts.DST])
