@@ -17,6 +17,7 @@ import hashlib
 import os
 import ssl
 import time
+from typing import Optional
 import warnings
 
 from oslo_context import context as oslo_context
@@ -33,20 +34,15 @@ from octavia.amphorae.drivers.keepalived import vrrp_rest_driver
 from octavia.common.config import cfg
 from octavia.common import constants as consts
 import octavia.common.jinja.haproxy.combined_listeners.jinja_cfg as jinja_combo
-import octavia.common.jinja.haproxy.split_listeners.jinja_cfg as jinja_split
 from octavia.common.jinja.lvs import jinja_cfg as jinja_udp_cfg
 from octavia.common.tls_utils import cert_parser
 from octavia.common import utils
-from octavia.db import api as db_apis
+from octavia.db import api as db_api
+from octavia.db import models as db_models
 from octavia.db import repositories as repo
-from octavia.network import data_models as network_models
 
 
 LOG = logging.getLogger(__name__)
-API_VERSION = consts.API_VERSION
-OCTAVIA_API_CLIENT = (
-    "Octavia HaProxy Rest Client/{version} "
-    "(https://wiki.openstack.org/wiki/Octavia)").format(version=API_VERSION)
 CONF = cfg.CONF
 
 
@@ -58,7 +54,6 @@ class HaproxyAmphoraLoadBalancerDriver(
         super().__init__()
         self.clients = {
             'base': AmphoraAPIClientBase(),
-            '0.5': AmphoraAPIClient0_5(),
             '1.0': AmphoraAPIClient1_0(),
         }
         self.cert_manager = stevedore_driver.DriverManager(
@@ -72,11 +67,6 @@ class HaproxyAmphoraLoadBalancerDriver(
             base_crt_dir=CONF.haproxy_amphora.base_cert_dir,
             haproxy_template=CONF.haproxy_amphora.haproxy_template,
             connection_logging=CONF.haproxy_amphora.connection_logging)
-        self.jinja_split = jinja_split.JinjaTemplater(
-            base_amp_path=CONF.haproxy_amphora.base_path,
-            base_crt_dir=CONF.haproxy_amphora.base_cert_dir,
-            haproxy_template=CONF.haproxy_amphora.haproxy_template,
-            connection_logging=CONF.haproxy_amphora.connection_logging)
         self.lvs_jinja = jinja_udp_cfg.LvsJinjaTemplater()
 
     def _get_haproxy_versions(self, amphora, timeout_dict=None):
@@ -113,7 +103,18 @@ class HaproxyAmphoraLoadBalancerDriver(
                 amphora.api_version = '0.5'
         LOG.debug('Amphora %s has API version %s',
                   amphora.id, amphora.api_version)
-        return list(map(int, amphora.api_version.split('.')))
+        api_version = list(map(int, amphora.api_version.split('.')))
+
+        if api_version[0] == 0 and api_version[1] <= 5:  # 0.5 or earlier
+            raise driver_except.AmpVersionUnsupported(
+                version=amphora.api_version)
+
+        return api_version
+
+    def check(self, amphora: db_models.Amphora,
+              timeout_dict: Optional[dict] = None):
+        """Check connectivity to the amphora."""
+        self._populate_amphora_api_version(amphora, timeout_dict)
 
     def update_amphora_listeners(self, loadbalancer, amphora,
                                  timeout_dict=None):
@@ -141,17 +142,8 @@ class HaproxyAmphoraLoadBalancerDriver(
         # Check which HAProxy version is on the amp
         haproxy_versions = self._get_haproxy_versions(
             amphora, timeout_dict=timeout_dict)
-        # Check which config style to use
-        api_version = self._populate_amphora_api_version(amphora)
-        if api_version[0] == 0 and api_version[1] <= 5:  # 0.5 or earlier
-            split_config = True
-            LOG.warning(
-                'Amphora %s for loadbalancer %s needs upgrade to single '
-                'process mode.', amphora.id, loadbalancer.id)
-        else:
-            split_config = False
-            LOG.debug('Amphora %s for loadbalancer %s is already in single '
-                      'process mode.', amphora.id, loadbalancer.id)
+        # Check if version is supported
+        self._populate_amphora_api_version(amphora)
 
         has_tcp = False
         certs = {}
@@ -168,10 +160,7 @@ class HaproxyAmphoraLoadBalancerDriver(
                     amphora, listener.id, timeout_dict=timeout_dict)
             else:
                 has_tcp = True
-                if split_config:
-                    obj_id = listener.id
-                else:
-                    obj_id = loadbalancer.id
+                obj_id = loadbalancer.id
 
                 try:
                     certs.update({
@@ -192,37 +181,27 @@ class HaproxyAmphoraLoadBalancerDriver(
                     certs.update(self._process_listener_pool_certs(
                         listener, amphora, obj_id))
 
-                    if split_config:
-                        config = self.jinja_split.build_config(
-                            host_amphora=amphora, listener=listener,
-                            haproxy_versions=haproxy_versions,
-                            client_ca_filename=certs[
-                                listener.client_ca_tls_certificate_id],
-                            client_crl=certs[listener.client_crl_container_id],
-                            pool_tls_certs=certs)
-                        self.clients[amphora.api_version].upload_config(
-                            amphora, listener.id, config,
-                            timeout_dict=timeout_dict)
-                        self.clients[amphora.api_version].reload_listener(
-                            amphora, listener.id, timeout_dict=timeout_dict)
-                    else:
-                        listeners_to_update.append(listener)
+                    listeners_to_update.append(listener)
                 except Exception as e:
                     LOG.exception('Unable to update listener %s due to '
                                   '"%s". Skipping this listener.',
                                   listener.id, str(e))
                     listener_repo = repo.ListenerRepository()
-                    listener_repo.update(db_apis.get_session(), listener.id,
-                                         provisioning_status=consts.ERROR,
-                                         operating_status=consts.ERROR)
+                    with db_api.session().begin() as session:
+                        listener_repo.update(session, listener.id,
+                                             provisioning_status=consts.ERROR,
+                                             operating_status=consts.ERROR)
 
-        if has_tcp and not split_config:
+        if has_tcp:
             if listeners_to_update:
                 # Generate HaProxy configuration from listener object
+                amp_details = self.clients[amphora.api_version].get_details(
+                    amphora)
                 config = self.jinja_combo.build_config(
                     host_amphora=amphora, listeners=listeners_to_update,
                     tls_certs=certs,
-                    haproxy_versions=haproxy_versions)
+                    haproxy_versions=haproxy_versions,
+                    amp_details=amp_details)
                 self.clients[amphora.api_version].upload_config(
                     amphora, loadbalancer.id, config,
                     timeout_dict=timeout_dict)
@@ -269,35 +248,20 @@ class HaproxyAmphoraLoadBalancerDriver(
         else:
             amphorae = [amphora]
 
-        timeout_dict = args[0]
-
         for amp in amphorae:
             if amp.status != consts.DELETED:
-                api_version = self._populate_amphora_api_version(
-                    amp, timeout_dict=timeout_dict)
-                # Check which config style to use
-                if api_version[0] == 0 and api_version[1] <= 5:
-                    # 0.5 or earlier
-                    LOG.warning(
-                        'Amphora %s for loadbalancer %s needs upgrade to '
-                        'single process mode.', amp.id, loadbalancer.id)
-                    for listener in loadbalancer.listeners:
+                self._populate_amphora_api_version(
+                    amp, timeout_dict=args[0])
+                has_tcp = False
+                for listener in loadbalancer.listeners:
+                    if listener.protocol in consts.LVS_PROTOCOLS:
                         getattr(self.clients[amp.api_version], func_name)(
                             amp, listener.id, *args)
-                else:
-                    LOG.debug(
-                        'Amphora %s for loadbalancer %s is already in single '
-                        'process mode.', amp.id, loadbalancer.id)
-                    has_tcp = False
-                    for listener in loadbalancer.listeners:
-                        if listener.protocol in consts.LVS_PROTOCOLS:
-                            getattr(self.clients[amp.api_version], func_name)(
-                                amp, listener.id, *args)
-                        else:
-                            has_tcp = True
-                    if has_tcp:
-                        getattr(self.clients[amp.api_version], func_name)(
-                            amp, loadbalancer.id, *args)
+                    else:
+                        has_tcp = True
+                if has_tcp:
+                    getattr(self.clients[amp.api_version], func_name)(
+                        amp, loadbalancer.id, *args)
 
     def reload(self, loadbalancer, amphora=None, timeout_dict=None):
         self._apply('reload_listener', loadbalancer, amphora, timeout_dict)
@@ -318,24 +282,9 @@ class HaproxyAmphoraLoadBalancerDriver(
             return
 
         # In case the listener is not UDP or SCTP, things get more complicated.
-        # We need to do this individually for each amphora in case some are
-        # using split config and others are using combined config.
         for amp in loadbalancer.amphorae:
             if amp.status != consts.DELETED:
-                api_version = self._populate_amphora_api_version(amp)
-                # Check which config style to use
-                if api_version[0] == 0 and api_version[1] <= 5:
-                    # 0.5 or earlier
-                    LOG.warning(
-                        'Amphora %s for loadbalancer %s needs upgrade to '
-                        'single process mode.', amp.id, loadbalancer.id)
-                    self.clients[amp.api_version].delete_listener(
-                        amp, listener.id)
-                else:
-                    LOG.debug(
-                        'Amphora %s for loadbalancer %s is already in single '
-                        'process mode.', amp.id, loadbalancer.id)
-                    self._combined_config_delete(amp, listener)
+                self._combined_config_delete(amp, listener)
 
     def _combined_config_delete(self, amphora, listener):
         # Remove the listener from the listener list on the LB before
@@ -414,26 +363,16 @@ class HaproxyAmphoraLoadBalancerDriver(
         return net_info
 
     def post_vip_plug(self, amphora, load_balancer, amphorae_network_config,
-                      vrrp_port=None, vip_subnet=None,
-                      additional_vip_data=None):
+                      vrrp_port, vip_subnet, additional_vip_data=None):
         if amphora.status != consts.DELETED:
             self._populate_amphora_api_version(amphora)
-            if vip_subnet is None:
-                vip_subnet = amphorae_network_config.get(amphora.id).vip_subnet
-            if vrrp_port is None:
-                port = amphorae_network_config.get(amphora.id).vrrp_port
-                mtu = port.network.mtu
-            else:
-                port = vrrp_port
-                mtu = port.network['mtu']
+            port = vrrp_port.to_dict(recurse=True)
+            mtu = port[consts.NETWORK][consts.MTU]
             LOG.debug("Post-VIP-Plugging with vrrp_ip %s vrrp_port %s",
-                      amphora.vrrp_ip, port.id)
+                      amphora.vrrp_ip, port[consts.ID])
             net_info = self._build_net_info(
-                port.to_dict(recurse=True), amphora.to_dict(),
+                port, amphora.to_dict(),
                 vip_subnet.to_dict(recurse=True), mtu)
-            if additional_vip_data is None:
-                additional_vip_data = amphorae_network_config.get(
-                    amphora.id).additional_vip_data
             for add_vip in additional_vip_data:
                 add_host_routes = [{'nexthop': hr.nexthop,
                                     'destination': hr.destination}
@@ -449,7 +388,7 @@ class HaproxyAmphoraLoadBalancerDriver(
             except exc.Conflict:
                 LOG.warning('VIP with MAC %(mac)s already exists on amphora, '
                             'skipping post_vip_plug',
-                            {'mac': port.mac_address})
+                            {'mac': port[consts.MAC_ADDRESS]})
 
     def post_network_plug(self, amphora, port, amphora_network_config):
         fixed_ips = []
@@ -466,10 +405,6 @@ class HaproxyAmphoraLoadBalancerDriver(
                      'fixed_ips': fixed_ips,
                      'mtu': port.network.mtu}
         if port.id == amphora.vrrp_port_id:
-            if isinstance(amphora_network_config,
-                          network_models.AmphoraNetworkConfig):
-                amphora_network_config = amphora_network_config.to_dict(
-                    recurse=True)
             # We have to special-case sharing the vrrp port and pass through
             # enough extra information to populate the whole VIP port
             net_info = self._build_net_info(
@@ -649,15 +584,15 @@ class HaproxyAmphoraLoadBalancerDriver(
                              req_read_timeout, conn_max_retries,
                              conn_retry_interval
         :type timeout_dict: dict
-        :returns: None if not found, the interface name string if found.
+        :returns: the interface name string if found.
+        :raises octavia.amphorae.drivers.haproxy.exceptions.NotFound:
+                No interface found on the amphora
+        :raises TimeOutException: The amphora didn't reply
         """
-        try:
-            self._populate_amphora_api_version(amphora, timeout_dict)
-            response_json = self.clients[amphora.api_version].get_interface(
-                amphora, ip_address, timeout_dict, log_error=False)
-            return response_json.get('interface', None)
-        except (exc.NotFound, driver_except.TimeOutException):
-            return None
+        self._populate_amphora_api_version(amphora, timeout_dict)
+        response_json = self.clients[amphora.api_version].get_interface(
+            amphora, ip_address, timeout_dict, log_error=False)
+        return response_json.get('interface', None)
 
 
 # Check a custom hostname
@@ -703,8 +638,10 @@ class AmphoraAPIClientBase(object):
             ip=ip,
             port=CONF.haproxy_amphora.bind_port)
 
-    def request(self, method, amp, path='/', timeout_dict=None,
-                retry_404=True, raise_retry_exception=False, **kwargs):
+    def request(self, method: str, amp: db_models.Amphora, path: str = '/',
+                timeout_dict: Optional[dict] = None,
+                retry_404: bool = True, raise_retry_exception: bool = False,
+                **kwargs):
         cfg_ha_amp = CONF.haproxy_amphora
         if timeout_dict is None:
             timeout_dict = {}
@@ -728,7 +665,9 @@ class AmphoraAPIClientBase(object):
         reqargs.update(kwargs)
         headers = reqargs.setdefault('headers', {})
 
-        headers['User-Agent'] = OCTAVIA_API_CLIENT
+        headers['User-Agent'] = (
+            f"Octavia HaProxy Rest Client/{amp.api_version} "
+            f"(https://wiki.openstack.org/wiki/Octavia)")
         self.ssl_adapter.uuid = amp.id
         exception = None
         # Keep retrying
@@ -794,123 +733,6 @@ class AmphoraAPIClientBase(object):
         return r.json()
 
 
-class AmphoraAPIClient0_5(AmphoraAPIClientBase):
-    def __init__(self):
-        super().__init__()
-
-        self.start_listener = functools.partial(self._action,
-                                                consts.AMP_ACTION_START)
-        self.reload_listener = functools.partial(self._action,
-                                                 consts.AMP_ACTION_RELOAD)
-
-        self.start_vrrp = functools.partial(self._vrrp_action,
-                                            consts.AMP_ACTION_START)
-        self.stop_vrrp = functools.partial(self._vrrp_action,
-                                           consts.AMP_ACTION_STOP)
-        self.reload_vrrp = functools.partial(self._vrrp_action,
-                                             consts.AMP_ACTION_RELOAD)
-
-    def upload_config(self, amp, listener_id, config, timeout_dict=None):
-        r = self.put(
-            amp,
-            'listeners/{amphora_id}/{listener_id}/haproxy'.format(
-                amphora_id=amp.id, listener_id=listener_id), timeout_dict,
-            data=config)
-        return exc.check_exception(r)
-
-    def _action(self, action, amp, listener_id, timeout_dict=None):
-        r = self.put(amp, 'listeners/{listener_id}/{action}'.format(
-            listener_id=listener_id, action=action), timeout_dict=timeout_dict)
-        return exc.check_exception(r)
-
-    def upload_cert_pem(self, amp, listener_id, pem_filename, pem_file):
-        r = self.put(
-            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
-                listener_id=listener_id, filename=pem_filename),
-            data=pem_file)
-        return exc.check_exception(r)
-
-    def get_cert_md5sum(self, amp, listener_id, pem_filename, ignore=tuple()):
-        r = self.get(
-            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
-                listener_id=listener_id, filename=pem_filename))
-        if exc.check_exception(r, ignore):
-            return r.json().get("md5sum")
-        return None
-
-    def delete_cert_pem(self, amp, listener_id, pem_filename):
-        r = self.delete(
-            amp, 'listeners/{listener_id}/certificates/{filename}'.format(
-                listener_id=listener_id, filename=pem_filename))
-        return exc.check_exception(r, (404,))
-
-    def update_cert_for_rotation(self, amp, pem_file):
-        r = self.put(amp, 'certificate', data=pem_file)
-        return exc.check_exception(r)
-
-    def delete_listener(self, amp, listener_id):
-        r = self.delete(
-            amp, 'listeners/{listener_id}'.format(listener_id=listener_id))
-        return exc.check_exception(r, (404,))
-
-    def get_info(self, amp, raise_retry_exception=False,
-                 timeout_dict=None):
-        r = self.get(amp, "info", raise_retry_exception=raise_retry_exception,
-                     timeout_dict=timeout_dict)
-        if exc.check_exception(r):
-            return r.json()
-        return None
-
-    def get_details(self, amp):
-        r = self.get(amp, "details")
-        if exc.check_exception(r):
-            return r.json()
-        return None
-
-    def get_all_listeners(self, amp):
-        r = self.get(amp, "listeners")
-        if exc.check_exception(r):
-            return r.json()
-        return None
-
-    def plug_network(self, amp, port):
-        r = self.post(amp, 'plug/network',
-                      json=port)
-        return exc.check_exception(r)
-
-    def plug_vip(self, amp, vip, net_info):
-        r = self.post(amp,
-                      'plug/vip/{vip}'.format(vip=vip),
-                      json=net_info)
-        return exc.check_exception(r)
-
-    def upload_vrrp_config(self, amp, config):
-        r = self.put(amp, 'vrrp/upload', data=config)
-        return exc.check_exception(r)
-
-    def _vrrp_action(self, action, amp, timeout_dict=None):
-        r = self.put(amp, 'vrrp/{action}'.format(action=action),
-                     timeout_dict=timeout_dict)
-        return exc.check_exception(r)
-
-    def get_interface(self, amp, ip_addr, timeout_dict=None, log_error=True):
-        r = self.get(amp, 'interface/{ip_addr}'.format(ip_addr=ip_addr),
-                     timeout_dict=timeout_dict)
-        return exc.check_exception(r, log_error=log_error).json()
-
-    def upload_udp_config(self, amp, listener_id, config, timeout_dict=None):
-        r = self.put(
-            amp,
-            'listeners/{amphora_id}/{listener_id}/udp_listener'.format(
-                amphora_id=amp.id, listener_id=listener_id), timeout_dict,
-            data=config)
-        return exc.check_exception(r)
-
-    def update_agent_config(self, amp, agent_config, timeout_dict=None):
-        r = self.put(amp, 'config', timeout_dict, data=agent_config)
-        return exc.check_exception(r)
-
-
 class AmphoraAPIClient1_0(AmphoraAPIClientBase):
     def __init__(self):
         super().__init__()
