@@ -297,6 +297,7 @@ SUPPORTED_TASKFLOW_ENGINE_TYPES = [
                  'non-dependent tasks simultaneously')]
 
 # Task/Flow constants
+ACCEPT = 'accept'
 ACTIVE_CONNECTIONS = 'active_connections'
 ADD_NICS = 'add_nics'
 ADD_SUBNETS = 'add_subnets'
@@ -324,6 +325,7 @@ CA_TLS_CERTIFICATE_ID = 'ca_tls_certificate_id'
 CIDR = 'cidr'
 CLIENT_CA_TLS_CERTIFICATE_ID = 'client_ca_tls_certificate_id'
 CLIENT_CRL_CONTAINER_ID = 'client_crl_container_id'
+CODE = 'code'
 COMPUTE_ID = 'compute_id'
 COMPUTE_OBJ = 'compute_obj'
 COMPUTE_ZONE = 'compute_zone'
@@ -338,8 +340,10 @@ DELTA = 'delta'
 DELTAS = 'deltas'
 DESCRIPTION = 'description'
 DESTINATION = 'destination'
+DETAIL = 'detail'
 DEVICE_OWNER = 'device_owner'
 ENABLED = 'enabled'
+ERRORS = 'errors'
 FAILED_AMP_VRRP_PORT_ID = 'failed_amp_vrrp_port_id'
 FAILED_AMPHORA = 'failed_amphora'
 FAILOVER_AMPHORA = 'failover_amphora'
@@ -412,6 +416,7 @@ REDIRECT_POOL = 'redirect_pool'
 REQ_CONN_TIMEOUT = 'req_conn_timeout'
 REQ_READ_TIMEOUT = 'req_read_timeout'
 REQUEST_ERRORS = 'request_errors'
+REQUEST_ID = 'request_id'
 ROLE = 'role'
 SECURITY_GROUPS = 'security_groups'
 SECURITY_GROUP_RULES = 'security_group_rules'
@@ -427,6 +432,7 @@ SUBNET_ID = 'subnet_id'
 TAGS = 'tags'
 TENANT_ID = 'tenant_id'
 TIMEOUT_DICT = 'timeout_dict'
+TITLE = 'title'
 TLS_CERTIFICATE_ID = 'tls_certificate_id'
 TLS_CONTAINER_ID = 'tls_container_id'
 TOPOLOGY = 'topology'
@@ -670,8 +676,6 @@ QUOTA_UNLIMITED = -1
 MIN_QUOTA = QUOTA_UNLIMITED
 MAX_QUOTA = 2000000000
 
-API_VERSION = '0.5'
-
 HAPROXY_BASE_PEER_PORT = 1025
 KEEPALIVED_JINJA2_UPSTART = 'keepalived.upstart.j2'
 KEEPALIVED_JINJA2_SYSTEMD = 'keepalived.systemd.j2'
@@ -796,8 +800,6 @@ RBAC_ROLES_DEPRECATED_REASON = (
 # PROVIDERS
 OCTAVIA = 'octavia'
 AMPHORAV2 = 'amphorav2'
-# Deprecated in Z, to be removed
-AMPHORAV1 = 'amphorav1'
 
 # systemctl commands
 DISABLE = 'disable'
