@@ -185,7 +185,6 @@ class TestNetworkTasks(base.TestCase):
         mock_lb_repo_get.return_value = lb_mock
         mock_driver.get_port.return_value = vrrp_port
         mock_driver.get_subnet.return_value = member_subnet
-        mock_driver.get_network.return_value = mgmt_net
         mock_driver.get_plugged_networks.return_value = [
             mgmt_interface,
             vrrp_interface,
@@ -231,9 +230,6 @@ class TestNetworkTasks(base.TestCase):
 
         mgmt_subnet = data_models.Subnet(
             id=self.mgmt_subnet_id, network_id=self.mgmt_net_id)
-        mgmt_net = data_models.Network(
-            id=self.mgmt_net_id,
-            subnets=[mgmt_subnet.id])
         mgmt_ip_address = mock.MagicMock()
         mgmt_interface = data_models.Interface(
             network_id=self.mgmt_net_id,
@@ -304,7 +300,6 @@ class TestNetworkTasks(base.TestCase):
                 fixed_ips=vrrp_port.fixed_ips)]
         mock_driver.get_port.return_value = vrrp_port
         mock_driver.get_subnet.return_value = vrrp_subnet
-        mock_driver.get_network.return_value = mgmt_net
 
         calc_delta = network_tasks.CalculateDelta()
 
@@ -407,10 +402,6 @@ class TestNetworkTasks(base.TestCase):
         mgmt2_subnet = data_models.Subnet(
             id=mgmt2_subnet_id,
             network_id=mgmt2_net_id)
-        mgmt2_net = data_models.Network(
-            id=mgmt2_net_id,
-            subnets=[mgmt2_subnet.id]
-        )
         mgmt2_interface = data_models.Interface(
             network_id=mgmt2_net_id,
             fixed_ips=[
@@ -419,7 +410,6 @@ class TestNetworkTasks(base.TestCase):
                     subnet_id=mgmt2_subnet_id,
                 )
             ])
-        mock_driver.get_network.return_value = mgmt2_net
         az = {
             constants.MANAGEMENT_NETWORK: mgmt2_net_id,
         }
@@ -439,7 +429,6 @@ class TestNetworkTasks(base.TestCase):
         # Test with one amp and one pool and one member, wrong network plugged
         # Delta should be one network/subnet to add and one to remove
         mock_driver.reset_mock()
-        mock_driver.get_network.return_value = mgmt_net
         member_mock = mock.MagicMock()
         member_mock.subnet_id = member_private_subnet.id
         pool_mock.members = [member_mock]
@@ -1411,7 +1400,7 @@ class TestNetworkTasks(base.TestCase):
         net_task.execute(listener)
         mock_driver.update_vip.assert_called_once_with(lb, for_delete=True)
 
-    @mock.patch('octavia.db.api.get_session', return_value='TEST')
+    @mock.patch('octavia.db.api.get_session')
     @mock.patch('octavia.db.repositories.AmphoraRepository.get')
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     def test_get_amphora_network_configs_by_id(
@@ -1430,8 +1419,8 @@ class TestNetworkTasks(base.TestCase):
 
         mock_driver.get_network_configs.assert_called_once_with(
             'mock load balancer', amphora='mock amphora')
-        mock_amp_get.assert_called_once_with('TEST', id=AMP_ID)
-        mock_lb_get.assert_called_once_with('TEST', id=LB_ID)
+        mock_amp_get.assert_called_once_with(mock_get_session(), id=AMP_ID)
+        mock_lb_get.assert_called_once_with(mock_get_session(), id=LB_ID)
 
     @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
     @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
@@ -1579,6 +1568,57 @@ class TestNetworkTasks(base.TestCase):
         mock_driver.unplug_aap_port.assert_called_once_with(
             LB.vip, self.db_amphora_mock, mockSubnet)
 
+    @mock.patch('octavia.db.repositories.AmphoraRepository.get')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
+    def test_revert_plug_vip_amphora_subnet_not_found(
+            self, mock_session, mock_lb_get, mock_get, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_lb_get.return_value = LB
+        mock_get.return_value = self.db_amphora_mock
+        mock_get_net_driver.return_value = mock_driver
+        net = network_tasks.PlugVIPAmphora()
+        amphora = {constants.ID: AMPHORA_ID,
+                   constants.LB_NETWORK_IP: IP_ADDRESS}
+        subnet = {constants.ID: SUBNET_ID}
+        err_msg = 'Subnet not found'
+        mock_driver.get_subnet.side_effect = net_base.SubnetNotFound(err_msg)
+        result = AMPS_DATA[0].to_dict()
+        net.revert(result, self.load_balancer_mock, amphora, subnet)
+        mock_driver.unplug_aap_port.assert_not_called()
+        network_tasks.LOG.error.assert_called_once_with(
+            'Failed to unplug AAP port for load balancer: %s. '
+            'Resources may still be in use for VRRP port: %s. '
+            'Due to error: %s',
+            self.load_balancer_mock[constants.LOADBALANCER_ID],
+            result[constants.VRRP_PORT_ID], err_msg
+        )
+
+    @mock.patch('octavia.db.repositories.AmphoraRepository.get')
+    @mock.patch('octavia.db.repositories.LoadBalancerRepository.get')
+    @mock.patch('octavia.db.api.get_session', return_value=_session_mock)
+    def test_revert_plug_vip_amphora_raise_db_error(
+            self, mock_session, mock_lb_get, mock_get, mock_get_net_driver):
+        mock_driver = mock.MagicMock()
+        mock_lb_get.return_value = LB
+        err_msg = 'Some Error'
+        mock_get.side_effect = Exception(err_msg)
+        net = network_tasks.PlugVIPAmphora()
+        amphora = {constants.ID: AMPHORA_ID,
+                   constants.LB_NETWORK_IP: IP_ADDRESS}
+        subnet = {constants.ID: SUBNET_ID}
+        result = AMPS_DATA[0].to_dict()
+        net.revert(result, self.load_balancer_mock, amphora, subnet)
+        mock_driver.unplug_aap_port.assert_not_called()
+        mock_lb_get.assert_not_called()
+        network_tasks.LOG.error.assert_called_once_with(
+            'Failed to unplug AAP port for load balancer: %s. '
+            'Resources may still be in use for VRRP port: %s. '
+            'Due to error: %s',
+            self.load_balancer_mock[constants.LOADBALANCER_ID],
+            result[constants.VRRP_PORT_ID], err_msg
+        )
+
     @mock.patch('octavia.controller.worker.v2.tasks.network_tasks.DeletePort.'
                 'update_progress')
     def test_delete_port(self, mock_update_progress, mock_get_net_driver):
@@ -1723,7 +1763,11 @@ class TestNetworkTasks(base.TestCase):
         # Test revert
         mock_driver.reset_mock()
 
-        net_task.revert([port_mock], vip_dict, VIP_SG_ID, AMP_ID,
+        # The execute path generates a port dict, so this will be the result
+        # passed into the revert method by Taskflow
+        port_dict = {constants.ID: PORT_ID}
+
+        net_task.revert(port_dict, vip_dict, VIP_SG_ID, AMP_ID,
                         additional_vips)
 
         mock_driver.delete_port.assert_called_once_with(PORT_ID)
@@ -1731,7 +1775,7 @@ class TestNetworkTasks(base.TestCase):
         # Test revert exception
         mock_driver.reset_mock()
 
-        net_task.revert([port_mock], vip_dict, VIP_SG_ID, AMP_ID,
+        net_task.revert(port_dict, vip_dict, VIP_SG_ID, AMP_ID,
                         additional_vips)
 
         mock_driver.delete_port.assert_called_once_with(PORT_ID)
